==> tovero-0.1.0.tar.gz: tovero-0.1.0/src/tovero/graphics/models/tovero.rb <==
#!/usr/bin/ruby
#
# tovero.rb
#
# Copyright 2012-2013 Roan Trail, Inc.
#
# This file is part of Tovero.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   (1) Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
#   (2) Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
#   (3) The name of the author may not be used to
#   endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

#
# Command line argument processing first
#

mode = ARGV[0]

if mode == 'edit'
  raytrace = false
elsif mode == 'render' or mode == nil
  raytrace = true
else
  abort "error, if an argument is supplied, it must be edit or render"
end

# These load the extension libraries
require 'libtovero_support_rb_0'
require 'libtovero_math_rb_0'
require 'libtovero_graphics_rb_0'

# These are like "using namespace" in C++
include Libtovero_support_rb_0
include Libtovero_math_rb_0
include Libtovero_graphics_rb_0

zero_m = Distance::meter * 0.0

stem_height = Distance::meter
thickness = Distance.new(0.2, Distance::meter)

background_color = Color.new(224, 212, 190, 0)

# Note: consistently setting the name of graphical entities makes
# it easier to work with the model in BRL-CAD, if necessary
stem = Cylinder.new(Point.new,
                    Vector.new(zero_m,
                               stem_height,
                               zero_m),
                    thickness,
                    "stem.s")

cross_bar_width = stem_height * (7.0 / 8.0)
cross_bar_base = (stem.base + stem.height) - Vector.new(cross_bar_width / 2.0,
                                                        zero_m,
                                                        zero_m)

cross_bar = Cylinder.new(cross_bar_base,
                         Vector.new(cross_bar_width,
                                    zero_m,
                                    zero_m),
                         thickness,
                         "cross_bar.s")

blaze_shape = Sphere.new(Point.new(zero_m,
                                   stem_height / 2.0 + thickness,
                                   stem_height + thickness / 2.0),
                         stem_height,
                         "blaze_shape.s")

tee = SolidCombination.new;
tee.name = "tee.r"
tee.<< stem
tee.<< cross_bar

main_shader = PhongShader::plastic
main_shader.shine = 5
main_color = Color.new(86, 59, 39, 0)
main_attributes = CombinationAttributes.new
main_attributes.shader = main_shader
main_attributes.color = main_color

main_tee = Operation.new(SolidOperand.new(tee),
                         SolidOperand.new(blaze_shape),
                         SolidOperator::Difference_op,
                         main_attributes,
                         "main_tee.r")

blaze_shader = PhongShader::plastic
blaze_shader.shine = 5
blaze_color = Color.new(172, 118, 78, 0)
blaze_attributes = CombinationAttributes.new
blaze_attributes.shader = blaze_shader
blaze_attributes.color = blaze_color

blaze = Operation.new(SolidOperand.new(tee),
                      SolidOperand.new(blaze_shape),
                      SolidOperator::Intersection_op,
                      blaze_attributes,
                      "blaze.r")

BC_DB = BCDatabase.new;
solid_list = BC_DB.top_solids;
solid_list << main_tee
solid_list << blaze

database_file = "tovero.g"
error = ErrorParam.new
success = BC_DB.write(database_file, true, error)
if (not success)
  puts("Could not write #{database_file}:")
  puts error.base.to_s
else
  if raytrace
    image_height = 4096
    image_width = 4096
    background = "#{background_color.red}/#{background_color.green}/#{background_color.blue}"

    # display raytraced model
    #   options to "rt":
    #     -A [ambient light fraction (0.0 - 1.0)]
    #     -J [jitter (1 = spatial)]
    #     -R [do not report overlaps]
    #     -l [lighting model number (7 = photon mapping)]
    #     -C [background color (R/G/B)]
    #     -w [width in pixels of rendering]
    #     -n [height in pixels (number of lines) of rendering]
    #     -M [read model/view info from stdin]
    #     <first positional arg> [database]
    #     <second positional arg> [entity in database to trace]
    #     (model/view info can be saved in mged with the "saveview" command)

    command = "rm -f #{database_file}.pix #{database_file}.png ; \
              rt -A0.5 -J1 -R -l7 -C#{background} -o #{database_file}.pix \
              -w#{image_width} -n#{image_height} -M #{database_file} blaze.r main_tee.r <<EOF\n\
              viewsize 4100;\n\
              orientation -0.0912 0.0935 0.00465 0.991;\n\
              eye_pt 1550 1930 7997;\n\
              start 0; clean; end;\n\
              EOF"
    %x[#{command}]
    # cleanup
    command = "pix-png -w#{image_width} -n#{image_height} -o #{database_file}.png #{database_file}.pix ; \
              rm -f #{database_file}.pix"
    %x[#{command}]
  else
    # [or] load the interactive editor
    command = "mged #{database_file}"
    %x[#{command}]
  end
end

==> tovero-0.2.0.tar.gz: tovero-0.2.0/src/tovero/graphics/models/tovero.rb <==
#!/usr/bin/ruby
#
# tovero.rb
#
# Copyright 2012-2014 Roan Trail, Inc.
#
# This file is part of Tovero.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   (1) Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
#   (2) Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
#   (3) The name of the author may not be used to
#   endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

#
# Command line argument processing first
#

mode = ARGV[0]

if mode == 'edit'
  raytrace = false
elsif mode == 'render' or mode == nil
  raytrace = true
else
  abort "error, if an argument is supplied, it must be edit or render"
end

# These load the extension libraries
require 'libtovero_support_rb_1'
require 'libtovero_math_rb_1'
require 'libtovero_graphics_rb_1'

# These are like "using namespace" in C++
include Libtovero_support_rb_1
include Libtovero_math_rb_1
include Libtovero_graphics_rb_1

zero_m = Distance::meter * 0.0

stem_height = Distance::meter
thickness = Distance.new(0.2, Distance::meter)

background_color = Color.new(224, 212, 190, 0)

# Note: consistently setting the name of graphical entities makes
# it easier to work with the model in BRL-CAD, if necessary
stem = Cylinder.new(Point.new,
                    Vector.new(zero_m,
                               stem_height,
                               zero_m),
                    thickness,
                    "stem.s")

cross_bar_width = stem_height * (7.0 / 8.0)
cross_bar_base = (stem.base + stem.height) - Vector.new(cross_bar_width / 2.0,
                                                        zero_m,
                                                        zero_m)

cross_bar = Cylinder.new(cross_bar_base,
                         Vector.new(cross_bar_width,
                                    zero_m,
                                    zero_m),
                         thickness,
                         "cross_bar.s")

blaze_shape = Sphere.new(Point.new(zero_m,
                                   stem_height / 2.0 + thickness,
                                   stem_height + thickness / 2.0),
                         stem_height,
                         "blaze_shape.s")

tee = SolidCombination.new("tee.c")
tee << stem
tee << cross_bar

main_shader = PhongShader::plastic
main_shader.shine = 5
main_color = Color.new(86, 59, 39, 0)
main_attributes = CombinationAttributes.new
main_attributes.shader = main_shader
main_attributes.color = main_color
main_attributes.is_part = true

main_tee = Operation.new(SolidOperand.new(tee),
                         SolidOperand.new(blaze_shape),
                         SolidOperator::Difference_op,
                         main_attributes,
                         "main_tee.r")

blaze_color = Color.new(172, 118, 78, 0)
blaze_attributes = CombinationAttributes.new(main_attributes)
blaze_attributes.color = blaze_color

blaze = Operation.new(SolidOperand.new(tee),
                      SolidOperand.new(blaze_shape),
                      SolidOperator::Intersection_op,
                      blaze_attributes,
                      "blaze.r")

BC_DB = BCDatabase.new
solid_list = BC_DB.top_solids
solid_list << main_tee
solid_list << blaze

database_file = "tovero.g"
error = ErrorParam.new
success = BC_DB.write(database_file, true, error)
if (not success)
  puts("Could not write #{database_file}:")
  puts error.base.to_s
else
  if raytrace
    image_height = 4096
    image_width = 4096
    background = "#{background_color.red}/#{background_color.green}/#{background_color.blue}"

    # raytrace
    #   options to "rt":
    #     -A [ambient light fraction (0.0 - 1.0)]
    #     -J [jitter (1 = spatial)]
    #     -R [do not report overlaps]
    #     -C [background color (R/G/B)]
    #     -o [output .pix file]
    #     -w [width in pixels of rendering]
    #     -n [height in pixels (number of lines) of rendering]
    #     -M [read model/view info from stdin]
    #     <first positional arg> [database]
    #     <second positional arg> [entity in database to trace]
    #     (model/view info can be saved in mged with the "saveview" command)

    command = "rm -f #{database_file}.pix #{database_file}.png ; \
              rt -A0.5 -J1 -R -C#{background} -o #{database_file}.pix \
              -w#{image_width} -n#{image_height} -M #{database_file} blaze.r main_tee.r <<EOF\n\
              viewsize 4100;\n\
              orientation -0.0912 0.0935 0.00465 0.991;\n\
              eye_pt 1550 1930 7997;\n\
              start 0; clean; end;\nEOF"
    result = %x[#{command}]

    if (result)
      # convert image and cleanup
      command = "pix-png -w#{image_width} -n#{image_height} -o #{database_file}.png #{database_file}.pix ; \
              rm -f #{database_file}.pix"
      %x[#{command}]
    end
  else
    # [or] load the interactive editor
    command = "mged #{database_file}"
    %x[#{command}]
  end
end
